import AsyncQueue from './AsyncQueue';
interface ResourceFactory<T, U = T> {
    create(): PromiseLike<T> | T;
    dispose?(resource: T): PromiseLike<void> | void;
    access?(resource: T): PromiseLike<U> | U;
}
declare class ResourcePooler<T, U> {
    #private;
    factory: ResourceFactory<T, U>;
    targetSize: number;
    currentSize: number;
    resourceQueue: AsyncQueue<T>;
    constructor({ factory }: {
        factory: ResourceFactory<T, U>;
    });
    use<O>(task: (resource: U) => Promise<O> | O, waitForDispose?: boolean): Promise<O>;
    resize(newSize: number): Promise<void>;
}
declare function createPool<T, U = T>(factory: ResourceFactory<T, U>, size?: number): Promise<ResourcePooler<T, U>>;
export { createPool, ResourcePooler };
