"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _ResourcePooler_resizing;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourcePooler = exports.createPool = void 0;
const AsyncQueue_1 = __importDefault(require("./AsyncQueue"));
class ResourcePooler {
    constructor({ factory }) {
        this.targetSize = 0;
        this.currentSize = 0;
        this.resourceQueue = new AsyncQueue_1.default();
        _ResourcePooler_resizing.set(this, false);
        this.factory = factory;
    }
    use(task, waitForDispose = false) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const resource = yield this.resourceQueue.dequeue();
            const accessor = (_a = this.factory.access) !== null && _a !== void 0 ? _a : ((r) => r);
            const accessed = (yield accessor(resource));
            const result = yield task(accessed);
            if (this.currentSize > this.targetSize) {
                // Get rid of excess resources
                const diposing = (_c = (_b = this.factory).dispose) === null || _c === void 0 ? void 0 : _c.call(_b, resource);
                if (waitForDispose)
                    yield diposing;
                this.currentSize--;
            }
            else {
                this.resourceQueue.enqueue(resource);
            }
            return result;
        });
    }
    resize(newSize) {
        return __awaiter(this, void 0, void 0, function* () {
            // prevent conflicting resizes
            if (__classPrivateFieldGet(this, _ResourcePooler_resizing, "f"))
                return;
            __classPrivateFieldSet(this, _ResourcePooler_resizing, true, "f");
            this.targetSize = newSize;
            while (newSize > this.currentSize) {
                this.currentSize++;
                const next = yield this.factory.create();
                this.resourceQueue.enqueue(next);
            }
            while (newSize < this.currentSize) {
                yield this.use(() => __awaiter(this, void 0, void 0, function* () { }), true);
            }
            __classPrivateFieldSet(this, _ResourcePooler_resizing, false, "f");
        });
    }
}
exports.ResourcePooler = ResourcePooler;
_ResourcePooler_resizing = new WeakMap();
function createPool(factory, size = 8) {
    return __awaiter(this, void 0, void 0, function* () {
        const pooler = new ResourcePooler({
            factory,
        });
        yield pooler.resize(size);
        return pooler;
    });
}
exports.createPool = createPool;
